#
# Makefile Template for Tessy
# Compiler: Keil ARM V6
# Debugger: Keil uVision5 Debugger
# Template Version: 2
#

# Defines to be generated by Tessy:
#
# BUFFER_SIZE           :=
# COMPILER_DEFINES      :=
# COMPILER_DIALECT      :=
# COMPILER_INCLUDES     :=
# COMPILER_INSTALL_PATH :=
# ENDIANESS             :=
# EXEFILE_EXTENSION     :=
# LINKER_FILE           :=
# MODULE_PATH           :=
# MODULE_PATH_DOS       :=
# OBJFILE_EXTENSION     :=
# PROJECTROOT           :=
# SCATTER_FILE          :=
# TARGET_INSTALL_PATH   :=
# TESSY_BIN             :=
# TESSY_SYS             :=
# TESTOBJECT            :=
# USER_LINK_OPTIONS     :=

#####################################################################
#BEGIN_DEFINES
#####################################################################

#
# Special macros for includes and defines
#
TS_DEF_SEPARATOR :=-D
define TS_DEFINE
$(subst ;, $(TS_DEF_SEPARATOR),$(1))
endef

TS_INC_SEPARATOR :=-I
define TS_INCLUDE
$(subst ;, $(TS_INC_SEPARATOR),$(1))
endef

TESSY_SYS_DOS   := $(shell ts_pathname -s '$(TESSY_SYS)')
MODULE_PATH_DOS := $(shell ts_pathname -s '$(MODULE_PATH)')
ARM_HOME        := $(shell ts_pathname -s '$(COMPILER_INSTALL_PATH)')
ARM_CC          := $(shell ts_pathname -s '$(ARM_HOME)/ARMCLANG')
ARM_LIB         := $(shell ts_pathname -s '$(ARM_CC)/lib')
ARM_CC_INC      := $(shell ts_pathname -s '$(ARM_CC)/include')
ARM_INC         := $(shell ts_pathname -s '$(CMSIS_PATH)/CMSIS/RTOS2/Include')
ARM_DEVICE_INC  := $(shell ts_pathname -s '$(CMSIS_PATH)/CMSIS/RTOS2/RTX/Include')
UVSC_HOME       := $(shell ts_pathname -s '$(TARGET_INSTALL_PATH)')

ifneq ($(SCATTER_FILE),)
SCATTER_FILE	 := --scatter "$(SCATTER_FILE)"
endif

TS_NAME         := TS_GNUTOOLS
IDE_NAME        := uvision_uvsc
COMPANY_NAME    := arm
PROCESSOR_NAME  := arm_v6

ifeq ("little","$(ENDIANESS)")
  TS_BYTEORDER := TS_HOST_BYTEORDER
  TS_BYTEORDER_OPTION := -mlittle-endian
endif
ifeq ("big","$(ENDIANESS)")
  TS_BYTEORDER := TS_NET_BYTEORDER
  TS_BYTEORDER_OPTION := -mbig-endian
endif
ifndef TS_BYTEORDER
  $(error Attribute 'Endianess' has invalid value '$(ENDIANESS)', only 'little' and 'big' are allowed.)
endif

all	 : check_success master slave

# Cygwin work around.
S_BUILDIN_TESSY_FLAG := $(wildcard $(MODULE_PATH)/tslows_builtin.tessy)

check_success :
ifneq ($(S_BUILDIN_TESSY_FLAG),)
	@echo *
	@echo * Obviously the last build failed. Cleaning up built-in files...
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@echo * Please, try again to generate a built-in target binary.
	@echo *
	@exit 1
endif

ifneq ($(COMPILER_DIALECT),)
COMPILER_DIALECT := -std=$(COMPILER_DIALECT)
endif

#
# MASTER & COMM Makefiles
# (you should not need to modify these settings)
#
M_COMM_MAKE    := $(TESSY_SYS)/make/master_$(IDE_NAME)_gnu_i386.mk
S_COMM_MAKE    := $(TESSY_SYS)/src/comm/comm_$(IDE_NAME)_$(PROCESSOR_NAME).mk

#
# SLAVE
# (You may change these settings to use your own compiler/linker options)
#
S_COMP_OPTIONS := -xc $(COMPILER_DIALECT) --target=$(TARGET_TRIPLE) -mthumb -mcpu=$(CPU) -mfpu=none -mfloat-abi=soft -g -c $(TS_BYTEORDER_OPTION) \
                  -fno-rtti -funsigned-char -fshort-enums -fshort-wchar -gdwarf-3 -O0 -ffunction-sections -MD \
                  -I$(ARM_CC_INC) -I$(ARM_INC) -I$(ARM_DEVICE_INC) -D_RTE_ -DARMCM4 \
                  -DTESSY -DTS_SLAVE -DTS_RVDS_ARM -DTS_ARM -DTS_HAVE_INT64 -DTS_HAVE_FLOAT64 \
                  -DTS_HAVE___int64 -DTS_AVOID_COMM_RETURN
S_LINK_OPTIONS := --cpu=$(CPU) --fpu=SoftVFP --load_addr_map_info --xref --callgraph --symbols --info sizes \
                  --info totals --info unused --info veneers --map \
                  --list "$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG).map" $(SCATTER_FILE)
S_INCLUDES     := -I$(TESSY_SYS_DOS)\include\tessy\comm -I$(MODULE_PATH_DOS)

S_CC           := $(ARM_CC)\bin\armclang.exe
S_ASM          := $(ARM_CC)\bin\armasm.exe
S_LINK         := $(ARM_CC)\bin\armlink.exe
S_STUB_OBJECT  := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs$(OBJFILE_EXTENSION)
S_UC_OBJECT    := $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr$(OBJFILE_EXTENSION)


#####################################################################
#END_DEFINES
#####################################################################

#####################################################################
#BEGIN_SOURCE
#####################################################################

#
# Template rules
#
$(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER)$(OBJFILE_EXTENSION): $(MODULE_PATH_DOS)\ts_src$$(SOURCENUMBER).c
	$(S_CC) $(S_INCLUDES) $(call TS_INCLUDE,$(USER_INCLUDES$$(SOURCENUMBER))) $(S_COMP_OPTIONS) \
		$(call TS_DEFINE,$(USER_DEFINES$$(SOURCENUMBER))$(S_COM_OPTIONS)) \
		$(USER_COMP_OPTIONS$$(SOURCENUMBER)) -o $@ -c "$<"


#####################################################################
#END_SOURCE
#####################################################################

#####################################################################
#BEGIN_TARGETS
#####################################################################

ifneq ($(S_GenerateBuiltinData),)
TARGET_FLAG := d
BUILTIN_OBJ := $(MODULE_PATH_DOS)\tslows_builtin$(OBJFILE_EXTENSION)
master :
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin$(OBJFILE_EXTENSION)
else
BUILTIN_OBJ :=
TARGET_FLAG := s
master : $(MODULE_PATH)/ts_$(TESTOBJECT)_m.exe
endif
slave  : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION)

#
# COMPILE USERCODE / STUBS
#
$(S_UC_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_usr.c
	@echo ***** Compiling Usercode *****
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o $@ -c $<

$(S_STUB_OBJECT) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_stubs.c
	@echo ***** Compiling Stubs *****
	$(S_CC) $(S_INCLUDES) $(S_COMP_OPTIONS) -o $@ -c $<

#
# COMPILE/LINK MASTER
#

include $(M_COMM_MAKE)


#
# COMPILE SLAVE COMMUNICATION MODULES

include $(S_COMM_MAKE)

#
# COMPILE/LINK SLAVE
#
$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) : $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s.c
	@echo *
	@echo ***** Compiling Slave *****
	$(S_CC) $(S_COMP_OPTIONS) -Wno-empty-body $(S_INCLUDES) -o $@ -c $<

#
# Handle Board Files
#
ifeq ("true","$(USE_BOARD_FILES)")
ifeq ("","$(INITOBJDIR)")
$(error Attribute 'Use Board Files' is set to true but 'InitObjDir' is empty.)
endif
INITOBJDIR := $(shell ts_pathname -s '$(INITOBJDIR)')
ifeq ("","$(INITSRCDIR)")
$(info Attribute 'InitSrcDir' is empty. Using objects for linking only.)
START_OBJS := $(wildcard $(INITOBJDIR)/*$(OBJFILE_EXTENSION))
else
INITSRCDIR := $(shell ts_pathname -s '$(INITSRCDIR)')
START_OBJS := $(patsubst $(INITSRCDIR)/%.c,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.c)) \
              $(patsubst $(INITSRCDIR)/%.s,$(INITOBJDIR)/%$(OBJFILE_EXTENSION),$(wildcard $(INITSRCDIR)/*.s))

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.c
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) -I$(INITSRCDIR) "$<" -o"$@"

$(INITOBJDIR)/%$(OBJFILE_EXTENSION) : $(INITSRCDIR)/%.s
	$(S_CC) $(S_COMP_OPTIONS) $(S_INCLUDES) "$<" -o"$@"
endif
ifeq ("","$(START_OBJS)")
$(warning Attribute 'Use Board Files' is set to true but no objects were found.)
endif
endif

#
# Objects to be delete after the build process.
#
TS_INTERMEDIATE_OBJS := $(BUILTIN_OBJ) $(S_OBJECTS) \
                        $(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_s$(OBJFILE_EXTENSION) \
                        $(S_SRC_OBJECTS) $(S_UC_OBJECT) $(S_STUB_OBJECT)
.INTERMEDIATE: $(TS_INTERMEDIATE_OBJS)

$(MODULE_PATH_DOS)\ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION) : $(START_OBJS) $(TS_INTERMEDIATE_OBJS)
	@echo *
	@echo ***** Linking Slave *****
	$(S_LINK) $(S_LINK_OPTIONS) $(USER_LINK_OPTIONS) -o $@ $(START_OBJS) $(TS_INTERMEDIATE_OBJS)
ifneq ($(S_GenerateBuiltinData),)
	@-ts_rm -f "$(MODULE_PATH)"/tslows_builtin.*
	@echo *
	@echo * Generated target binary '$(shell ts_pathname -w $(MODULE_PATH)/ts_$(TESTOBJECT)_$(TARGET_FLAG)$(EXEFILE_EXTENSION))' containing builtin data.
	@echo *
endif

#####################################################################
#END_TARGETS
#####################################################################

